'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      CallExpression: function CallExpression(path, _ref2) {
        var _ref2$opts$excludes = _ref2.opts.excludes,
            excludes = _ref2$opts$excludes === undefined ? [] : _ref2$opts$excludes;

        if (!isJSXCall(path)) return;
        if (isExcludes(path, [ErrorBoundary])) return;
        if (isExcludes(path, excludes)) removeErrorBoundary(path);
        if (isExcludes(path.parentPath, excludes.concat([ErrorBoundary]))) return;

        var eNode = t.callExpression(t.memberExpression(t.identifier('React'), t.identifier('createElement')), path.node.arguments);

        var key = getKey(path);
        path.node.arguments = [t.identifier(ErrorBoundary), key ? t.objectExpression([key]) : t.nullLiteral(), eNode];
      }
    }
  };
};

var _ = require('lodash');
var ErrorBoundary = 'ErrorBoundary';


var isJSXCall = function isJSXCall(path) {
  var className = _.get(path, 'node.callee.object.name');
  var propName = _.get(path, 'node.callee.property.name');
  return className === 'React' && propName === 'createElement';
};

var isExcludes = function isExcludes(path, excludes) {
  var name = _.get(path, 'node.arguments[0].name');
  return excludes.includes(name);
};

var removeErrorBoundary = function removeErrorBoundary(path) {
  var childName = _.get(path, 'node.arguments[2].arguments[0].name');
  if (childName === ErrorBoundary) {
    path.node.arguments[2] = path.node.arguments[2].arguments[2];
  }
};

var getKey = function getKey(path) {
  var props = _.get(path, 'node.arguments[1].properties', []);
  var key = props.find(function (p) {
    return p.key.name === 'key';
  });
  return key;
};